import { parseImageSize } from './helpers/parse_image_size';
function image_with_size(md, options) {
    return function (state, silent) {
        let attrs, code, label, labelEnd, labelStart, pos, ref, res, title, width = '', height = '', token, tokens, start, href = '', oldPos = state.pos, max = state.posMax;
        if (state.src.charCodeAt(state.pos) !== 0x21) {
            return false;
        }
        if (state.src.charCodeAt(state.pos + 1) !== 0x5b) {
            return false;
        }
        labelStart = state.pos + 2;
        labelEnd = md.helpers.parseLinkLabel(state, state.pos + 1, false);
        if (labelEnd < 0) {
            return false;
        }
        pos = labelEnd + 1;
        if (pos < max && state.src.charCodeAt(pos) === 0x28) {
            pos++;
            for (; pos < max; pos++) {
                code = state.src.charCodeAt(pos);
                if (code !== 0x20 && code !== 0x0a) {
                    break;
                }
            }
            if (pos >= max) {
                return false;
            }
            start = pos;
            res = md.helpers.parseLinkDestination(state.src, pos, state.posMax);
            if (res.ok) {
                href = state.md.normalizeLink(res.str);
                if (state.md.validateLink(href)) {
                    pos = res.pos;
                }
                else {
                    href = '';
                }
            }
            start = pos;
            for (; pos < max; pos++) {
                code = state.src.charCodeAt(pos);
                if (code !== 0x20 && code !== 0x0a) {
                    break;
                }
            }
            res = md.helpers.parseLinkTitle(state.src, pos, state.posMax);
            if (pos < max && start !== pos && res.ok) {
                title = res.str;
                pos = res.pos;
                for (; pos < max; pos++) {
                    code = state.src.charCodeAt(pos);
                    if (code !== 0x20 && code !== 0x0a) {
                        break;
                    }
                }
            }
            else {
                title = '';
            }
            if (pos - 1 >= 0) {
                code = state.src.charCodeAt(pos - 1);
                if (code === 0x20) {
                    res = parseImageSize(state.src, pos, state.posMax);
                    if (res.ok) {
                        width = res.width;
                        height = res.height;
                        pos = res.pos;
                        for (; pos < max; pos++) {
                            code = state.src.charCodeAt(pos);
                            if (code !== 0x20 && code !== 0x0a) {
                                break;
                            }
                        }
                    }
                }
            }
            if (pos >= max || state.src.charCodeAt(pos) !== 0x29) {
                state.pos = oldPos;
                return false;
            }
            pos++;
        }
        else {
            if (typeof state.env.references === 'undefined') {
                return false;
            }
            for (; pos < max; pos++) {
                code = state.src.charCodeAt(pos);
                if (code !== 0x20 && code !== 0x0a) {
                    break;
                }
            }
            if (pos < max && state.src.charCodeAt(pos) === 0x5b) {
                start = pos + 1;
                pos = md.helpers.parseLinkLabel(state, pos);
                if (pos >= 0) {
                    label = state.src.slice(start, pos++);
                }
                else {
                    pos = labelEnd + 1;
                }
            }
            else {
                pos = labelEnd + 1;
            }
            if (!label) {
                label = state.src.slice(labelStart, labelEnd);
            }
            ref = state.env.references[md.utils.normalizeReference(label)];
            if (!ref) {
                state.pos = oldPos;
                return false;
            }
            href = ref.href;
            title = ref.title;
        }
        if (!silent) {
            state.pos = labelStart;
            state.posMax = labelEnd;
            const newState = new state.md.inline.State(state.src.slice(labelStart, labelEnd), state.md, state.env, (tokens = []));
            newState.md.inline.tokenize(newState);
            token = state.push('image', 'img', 0);
            token.attrs = attrs = [
                ['src', href],
                ['alt', ''],
            ];
            token.children = tokens;
            if (title) {
                attrs.push(['title', title]);
            }
            if (width !== '') {
                attrs.push(['width', width]);
            }
            if (height !== '') {
                attrs.push(['height', height]);
            }
        }
        state.pos = pos;
        state.posMax = max;
        return true;
    };
}
export function ImageSizePlugin(md, options) {
    md.inline.ruler.before('emphasis', 'image', image_with_size(md, options));
}
//# sourceMappingURL=index.js.map